<?php

    /*
        Plugin Name: Rapidexpress
        Plugin URI: https://plat-on.es
        Description: Plugin conexion con Rapidexpress
        Version: 1.8
        Author: Plat-on
        Author URI: https://plat-on.es
        License: GPL2
    */

    if ( ! defined( 'ABSPATH' ) ) {
        exit; // Exit if accessed directly
    }

    class Rapidexpress_Plugin {
        
        public function __construct() {
            // Hook into the admin menu
            add_action( 'admin_menu', array( $this, 'create_plugin_settings_page' ) );
            add_action( 'admin_init', array( $this, 'setup_sections' ) );
            add_action( 'admin_init', array( $this, 'setup_fields' ) );
            add_action( 'add_meta_boxes', array($this, 'add_meta_boxesws') );
        }

        public function create_plugin_settings_page() {
            // Add the menu item and page
            $page_title = 'Rapid Express Settings Page';
            $menu_title = 'RapidExpress';
            $capability = 'manage_options';
            $slug = 'rapidexpress';
            $callback = array( $this, 'plugin_settings_page_content' );
            $icon = 'dashicons-admin-plugins';
            $position = 100;
        
            add_menu_page( $page_title, $menu_title, $capability, $slug, $callback, $icon, $position );
        }

        public function plugin_settings_page_content() {
            if( isset($_POST['updated']) && $_POST['updated'] === 'true' ){
                $this->handle_form();
            } ?>
            <div class="wrap">
                <h2>Configuración plugin Rapid Express</h2>
                <form method="POST">
                    <input type="hidden" name="updated" value="true" />
                    <table class="form-table">
                        <tbody>
                            <?php
                                    wp_nonce_field( 'rapidexpress_update', 'rapidexpress_form' );
                                    settings_fields( 'rapidexpress' );
                                    do_settings_sections( 'rapidexpress' );
                                    submit_button();
                            ?>
                        </tbody>
                    </table>
                </form>
            </div> <?php
        }

        public function handle_form() {
            
            if(
                ! isset( $_POST['rapidexpress_form'] ) ||
                ! wp_verify_nonce( $_POST['rapidexpress_form'], 'rapidexpress_update' )
            ){ ?>
                <div class="error">
                   <p>Sorry, your nonce was not correct. Please try again.</p>
                </div> <?php
                exit;
            } else {

                update_option( 'rapid_token', $_POST['rapid_token'] );
                update_option( 'rapid_recogida_local', $_POST['rapid_recogida_local'] );


                $serviciosRapidContratados = $this->obtenerServiciosRapid($_POST['rapid_token']);
                $serviciosRapidContratados = json_decode($serviciosRapidContratados, true);

                $fields = [];
                
                foreach($serviciosRapidContratados["servicios"] as $key => $servicioRapidContratado){
                    $fields[$servicioRapidContratado["id"]] = $servicioRapidContratado["nombre"];
                }

                $field = array(
                    'uid' => 'rapid_servicios',
                    'label' => 'Servicios contratados',
                    'section' => 'our_first_section',
                    'type' => 'multiple-select',
                    'options' => $fields,
                    'supplemental' => 'Listado de servicios contratados con Rapid Express',
                    'default' => 0,
                    'disabled' => true 
                );

                add_settings_field( $field['uid'], $field['label'], array( $this, 'field_callback' ), 'rapidexpress', $field['section'], $field );
                register_setting( 'rapidexpress', $field['uid'] );
                update_option( 'rapid_servicios', $fields );            

            }
        }

        public function setup_sections() {
            add_settings_section( 'our_first_section', 'Configuración básica', array( $this, 'section_callback' ), 'rapidexpress' );
        }

        public function section_callback( $arguments ) {
            switch( $arguments['id'] ){
                case 'our_first_section':
                    break;
            }
        }

        public function setup_fields() {
            $fields = array(
                array(
                    'uid' => 'rapid_token',
                    'label' => 'Token',
                    'section' => 'our_first_section',
                    'type' => 'text',
                    'options' => false,
                    'placeholder' => 'Token de Rapid Express',
                    'default' => ''
                ),
               
                array(
                    'uid' => 'rapid_recogida_local',
                    'label' => 'Recogida Local',
                    'section' => 'our_first_section',
                    'type' => 'select',
                    'options' => array(
                        1 => 'Si',
                        0 => 'No',
                    ),
                    'default' => 1,
                    'disabled' => true 
                )
            );

            if(get_option('rapid_servicios')){
                $fields[] = array(
                    'uid' => 'rapid_servicios',
                    'label' => 'Servicios contratados',
                    'section' => 'our_first_section',
                    'type' => 'multiple-select',
                    'options' => get_option('rapid_servicios'),
                    'supplemental' => 'Listado de servicios contratados con Rapid Express.<br>No pueden ser modificados desde aqui.',
                    'default' => 0,
                    'disabled' => true 
                );
            }

            foreach( $fields as $field ){
                add_settings_field( $field['uid'], $field['label'], array( $this, 'field_callback' ), 'rapidexpress', $field['section'], $field );
                register_setting( 'rapidexpress', $field['uid'] );
            }
        }
        

        public function field_callback( $arguments ) {
            
            $value = get_option( $arguments['uid'] ); 
            if( ! $value ) { 
                $value = $arguments['default']; 
            }

            switch( $arguments['type'] ){
                case 'text':
                    printf( '<input name="%1$s" id="%1$s" type="%2$s" placeholder="%3$s" value="%4$s" />', $arguments['uid'], $arguments['type'], $arguments['placeholder'], $value );
                    break;
                case 'textarea': 
                    printf( '<textarea name="%1$s" id="%1$s" placeholder="%2$s" rows="5" cols="50">%3$s</textarea>', $arguments['uid'], $arguments['placeholder'], $value );
                    break;
                case 'select': 
                    if( ! empty ( $arguments['options'] ) && is_array( $arguments['options'] ) ){
                        $options_markup = '';
                        foreach( $arguments['options'] as $key => $label ){
                            $options_markup .= sprintf( '<option value="%s" %s>%s</option>', $key, selected( $value, $key, false ), $label );
                        }
                        printf( '<select name="%1$s" id="%1$s" '.($arguments["disabled"] ? "disabled" : "").' >%2$s</select>', $arguments['uid'], $options_markup );
                    }
                    break;
                case 'multiple-select':
                    if( ! empty ( $arguments['options'] ) && is_array( $arguments['options'] ) ){
                        $options_markup = '';
                        foreach( $arguments['options'] as $key => $label ){
                            $options_markup .= sprintf( '<option value="%s" %s>%s</option>', $key, '', $label );
                        }
                        printf( '<select name="%1$s" id="%1$s" multiple="multiple"'.($arguments["disabled"] ? "disabled" : "").' style="height:200px">%2$s</select>', $arguments['uid'], $options_markup );                    
                    }
                    break;
            }

            if( isset($arguments['helper']) && $helper = $arguments['helper'] ){
                printf( '<span class="helper"> %s</span>', $helper );
            }

            if( isset($arguments['supplemental']) && $supplimental = $arguments['supplemental'] ){
                printf( '<p class="description">%s</p>', $supplimental );
            }
            
        }

        public function obtenerServiciosRapid($token){
            
            $return = "";
            $data = array("apiKey" => $token);
            $data_string = json_encode($data);
            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => "https://portal.rapidexpress.es/api/servicios.php",
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => "",
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_CUSTOMREQUEST => "POST",
                CURLOPT_POSTFIELDS => array("data"=>$data_string),
                CURLOPT_HTTPHEADER => array(
                    "cache-control: no-cache"
                ),
            ));

            $response = curl_exec($curl);
            $err = curl_error($curl);
            curl_close($curl);
            if ($err) {
                $return = "cURL Error #:" . $err;
            } else {
                $return = $response;
            }        
            return $return;
        }


        public function add_meta_boxesws( $order ){
            add_meta_box( 'custom_order_meta_box', __( 'Rapid Express' ),
                    'custom_metabox_content', 'shop_order', 'normal', 'default');
            function custom_metabox_content( $order ){

                $order = wc_get_order($order);    
                $total_weight = 0;        
                foreach( $order->get_items() as $item_id => $product_item ){
                    $quantity = $product_item->get_quantity(); // get quantity
                    $product = $product_item->get_product(); // get the WC_Product object
                    $product_weight = $product->get_weight(); // get the product weight
                    $total_weight += floatval( $product_weight * $quantity );
                }
                if($order->get_shipping_first_name() == "" && $order->get_shipping_address_1() == ""){
                    $contacto = $order->get_address()['first_name'].' '.$order->get_address()['last_name'];
                    $direccion = $order->get_address()['address_1'].' '.$order->get_address()['address_2'];
                    $cp = $order->get_address()['postcode'];
                    $ciudad = $order->get_address()['city'];
                    $telefono = $order->get_address()['phone'];
                    $pais = $order->get_address()['country'];
                }else{
                    $contacto = $order->get_shipping_first_name().' '.$order->get_shipping_last_name();
                    $direccion = $order->get_shipping_address_1().' '.$order->get_shipping_address_2();
                    $cp = $order->get_shipping_postcode();
                    $ciudad = $order->get_shipping_city();
                    $telefono = $order->get_shipping_phone();
                    $pais = $order->get_shipping_country();
                }      
                               
                $email = $order->get_address()['email'];                  
                $referencia = 1;
                $kilos = $total_weight;
                $bultos = 1;
                $tipo = 'PAQ';
                $obervaciones = '';
                $valorAsegurado = 0;
                $valorReembolso = 0;

                // The main address pieces:
                $store_address     = get_option( 'woocommerce_store_address' ).' '.get_option( 'woocommerce_store_address_2' );
                $store_city        = get_option( 'woocommerce_store_city' );
                $store_postcode    = get_option( 'woocommerce_store_postcode' );

                // The country/state
                $store_raw_country = get_option( 'woocommerce_default_country' );

                // Split the country/state
                $split_country = explode( ":", $store_raw_country );

                // Country and state separated:
                $store_country = $split_country[0];
                $admin_email = get_option('admin_email');

                echo "<div id='datosDestinatario'>";
                    echo "<h2><b>Datos del destinatario</b></h2>";
                    woocommerce_form_field(
                        'rapid_destino_nombre', array(
                            'id' => 'rapid_destino_nombre',
                            'type' => 'text',
                            'class' =>array('form-row form-row-first'),
                            'label' => __('Nombre: '),
                            'required' => true,
                        ), $contacto
                    );

                    woocommerce_form_field(
                        'rapid_destino_contacto', array(
                            'id' => 'rapid_destino_contacto',
                            'type' => 'text',
                            'class' =>array('form-row form-row-last'),
                            'label' => __('Contacto: '),
                            'required' => true,             
                        ), $contacto
                    );

                    woocommerce_form_field(
                        'rapid_destino_direccion', array(
                            'id' => 'rapid_destino_direccion',
                            'type' => 'text',
                            'class' =>array('form-row form-row-wide'),
                            'label' => __('Direccion: '),
                            'required' => true,
                        ), $direccion
                    );

                    woocommerce_form_field(
                        'rapid_destino_cp', array(
                            'id' => 'rapid_destino_cp',
                            'type' => 'number',
                            'class' =>array('form-row form-row-first'),
                            'label' => __('Código Postal: '),
                            'required' => true,
                        ), $cp
                    );

                    woocommerce_form_field(
                        'rapid_destino_ciudad', array(
                            'id' => 'rapid_destino_ciudad',
                            'type' => 'text',
                            'class' =>array('form-row form-row-last'),
                            'label' => __('Ciudad: '),
                            'required' => true,
                        ), $ciudad
                    );

                    woocommerce_form_field(
                        'rapid_destino_pais', array(
                            'id' => 'rapid_destino_pais',
                            'type' => 'country',
                            'class' =>array('form-row form-row-first'),
                            'label' => __('Pais: '),
                            'required' => true,
                        ), $pais
                    );

                    woocommerce_form_field(
                        'rapid_destino_telefono', array(
                            'id' => 'rapid_destino_telefono',
                            'type' => 'text',
                            'class' =>array('form-row form-row-last'),
                            'label' => __('Teléfono: '),
                            'required' => true,
                        ), $telefono
                    );

                    woocommerce_form_field(
                        'rapid_destino_email', array(
                            'id' => 'rapid_destino_email',
                            'type' => 'email',
                            'class' =>array('form-row form-row-first'),
                            'label' => __('Email: '),
                        ), $email
                    );

                    woocommerce_form_field(
                        'rapid_destino_kilos', array(
                            'id' => 'rapid_destino_kilos',
                            'type' => 'number',
                            'class' =>array('form-row form-row-last'),
                            'label' => __('Kilos: '),
                            'required' => true,
                        ), $kilos
                    );

                    woocommerce_form_field(
                        'rapid_destino_bultos', array(
                            'id' => 'rapid_destino_bultos',
                            'type' => 'number',
                            'class' =>array('form-row form-row-first'),
                            'label' => __('Bultos: '),
                            'required' => true,                 
                        ), $bultos
                    );

                    woocommerce_form_field(
                        'rapid_destino_tipo', array(
                            'id' => 'rapid_destino_tipo',
                            'type' => 'select',
                            'class' =>array('form-row form-row-last'),
                            'label' => __('Tipo: '),
                            'required' => true,
                            'options' => array( 'PAQ' => 'PAQ' , 'DOC' => 'DOC'),
                        ), 'PAQ'
                    );

                    woocommerce_form_field(
                        'rapid_destino_referencia', array(
                            'id' => 'rapid_destino_referencia',
                            'type' => 'text',
                            'class' =>array('form-row form-row-wide'),
                            'label' => __('Referencia: '),
                        )
                    );

                    woocommerce_form_field(
                        'rapid_destino_observaciones', array(
                            'id' => 'rapid_destino_observaciones',
                            'type' => 'textarea',
                            'class' =>array('form-row form-row-wide'),
                            'label' => __('Observaciones: '),
                        )
                    );

                    woocommerce_form_field(
                        'rapid_destino_valor_asegurado', array(
                            'id' => 'rapid_destino_valor_asegurado',
                            'type' => 'number',
                            'class' =>array('form-row form-row-first'),
                            'label' => __('Valor Asegurado: '),
                        ), 0
                    );
                    
                    woocommerce_form_field(
                        'rapid_destino_valor_reembolso', array(
                            'id' => 'rapid_destino_valor_reembolso',
                            'type' => 'number',
                            'class' =>array('form-row form-row-last'),
                            'label' => __('Valor Reembolso: '),
                        ), 0
                    );

                    echo "</div>";                
                    echo "<hr style='display: inline-block;width: 100%;'><h2 style='display: inline-block;width: 100%;'><b>Datos Rapid Express</b></h2>";
                    echo "<span id='productoRapidExpress'>";

                        woocommerce_wp_select( array(
                            'id' => 'rapid_producto',
                            'label' => 'Producto Rapidexpress: ',
                            'options' =>  ['' => 'Cargando servicios contratados con Rapid Express ...'],
                            'wrapper_class' => 'form-row-wide',
                            'custom_attributes' => array('disabled' => 'disabled'),
                        ) );

                    echo "</span>";

                    woocommerce_wp_text_input( array(
                        'id' => 'rapid_envio',
                        'label' => 'Nº Envio Rapid Express: ',
                        'value' => get_post_meta($order->get_id(),'rapid_envio',true),
                        'wrapper_class' => 'form-row form-row-first',
                        'custom_attributes' => array('readonly' => 'readonly'),
                    ) );

                    
                    woocommerce_wp_text_input( array(
                        'id' => 'rapid_albaran',
                        'label' => 'Albarán Rapid Express: ',
                        'value' => get_post_meta($order->get_id(),'rapid_albaran',true),
                        'wrapper_class' => 'form-row form-row-last',
                        'custom_attributes' => array('readonly' => 'readonly'),
                    ) );

                    echo "<hr style='display: inline-block;width: 100%;'>";
                    $token = get_option( 'rapid_token', $_POST['rapid_token'] );
                ?>

                <?php 

                    if($order->get_meta('rapid_envio') == ""){
                        echo "<div style='height: 40px;'>";
                        echo "<span style='float:left; width: 30%;'>";
                        echo "<button type='button' class='button add-line-item' id='generarEnvioRapid' >Petición Recogida</button>";
                        echo "</span>";
                        echo "</div>";
                    }else{
                        echo "<button type='button' class='button add-line-item' id='obtenerAlbaran' >Descargar Albarán</button>";
                    }
                
                ?>
                
                <?php
                
                    add_action( 'wp_enqueue_scripts', 'ajax_test_enqueue_scripts' );
                    function ajax_test_enqueue_scripts() {
                        wp_enqueue_script( 'test', plugins_url( '/ajax.js', __FILE__ ), array('jquery'), '1.0', true );
                    }

                ?>
                <script type="text/javascript" >

                    jQuery(document).ready(function($) {

                        if($('#rapid_envio').val() != ''){
                            $('#datosRecogida').hide();
                            $('#datosDestinatario').hide();
                            $("#productoRapidExpress").hide();
                        }

                        var data = { 'function': 'solicitarServiciosRapid' };

                        $.ajax({
                            type: "POST",
                            url: '../wp-content/plugins/rapidexpress_wordpress/functions.php',
                            data: data,
                            success: function(success){
                                $("#rapid_producto").html('');
                                $("#rapid_producto").append('<option value="">Selecciona el servicio de Rapid Express</option>');
                                $.each(JSON.parse(success)['servicios'], function(id,value){
                                    $("#rapid_producto").append('<option value="'+value['id']+'">'+value['nombre']+'</option>');
                                });
                                $("#rapid_producto").prop("disabled", false);
                            },
                        });

                        function checkFields(e){

                            var result = true;
                            var errorsDestino = [];
                            var errorServicio = false;
                            
                            if(e.target.id == 'generarEnvioRapid' || e.target.id == 'solicitarSoloAlbaran'){
                                if($('#rapid_destino_nombre').val() == ''){ errorsDestino.push('nombre'); result = false;}
                                if($('#rapid_destino_contacto').val() == ''){ errorsDestino.push('contacto'); result = false;}
                                if($('#rapid_destino_direccion').val() == ''){ errorsDestino.push('dirección'); result = false;}
                                if($('#rapid_destino_cp').val() == ''){ errorsDestino.push('código postal'); result = false;}
                                if($('#rapid_destino_ciudad').val() == ''){ errorsDestino.push('ciudad'); result = false;}
                                if($('#rapid_destino_pais').val() == ''){ errorsDestino.push('pais'); result = false;}
                                if($('#rapid_destino_telefono').val() == ''){ errorsDestino.push('teléfono'); result = false;}
                                if($('#rapid_destino_kilos').val() == ''){ errorsDestino.push('kilos'); result = false;}
                                if($('#rapid_destino_bultos').val() == ''){ errorsDestino.push('bultos'); result = false;}
                            }

                            if($('#rapid_producto').val() == ''){ errorServicio = true; result = false;}
                            if(errorsDestino.length || errorServicio){

                                var textoAlert = 'Falta rellenar los siguientes datos\n';                               
                                if(errorsDestino.length > 0){
                                    textoAlert += '- Destino:\n';
                                    $.each( errorsDestino, function( key, value ) { textoAlert += value+'\n'; });
                                }

                                if(errorServicio){
                                    textoAlert += '- Rapid Express:\n Producto Rapid Express';
                                }
                                alert(textoAlert);
                            }

                            return result;
                        }                

                        jQuery('#generarEnvioRapid').on('click',function(e){

                            event.target.disabled = true;
                            if(!checkFields(e)){ 
                                e.preventDefault();
                                event.target.disabled = false;
                                return false;
                            }

                            var destino = {
                                'nombre': $('#rapid_destino_nombre').val(),
                                'contacto': $('#rapid_destino_contacto').val(),
                                'direccion': $('#rapid_destino_direccion').val(),
                                'cp': $('#rapid_destino_cp').val(),
                                'ciudad': $('#rapid_destino_ciudad').val(),
                                'pais': $('#rapid_destino_pais').val(),
                                'telefono': $('#rapid_destino_telefono').val(),
                                'email': $('#rapid_destino_email').val(),
                                'kilos': $('#rapid_destino_kilos').val(),
                                'bultos': $('#rapid_destino_bultos').val(),
                                'tipo': $('#rapid_destino_tipo').val(),
                                'referencia': $('#rapid_destino_referencia').val(),
                                'observaciones': $('#rapid_destino_observaciones').val(),
                                'valor_asegurado': $('#rapid_destino_valor_asegurado').val(),
                                'valor_reembolso': $('#rapid_destino_valor_reembolso').val(),
                            };
                            
                            var recogida = {
                                'nombre': '',
                                'contacto': '',
                                'direccion': '<?php echo get_option( "woocommerce_store_address" ); ?>',
                                'cp': '<?php echo get_option( "woocommerce_store_postcode" ); ?>',
                                'ciudad': '<?php echo get_option( "woocommerce_store_city" ); ?>',
                                'pais': '',
                                'telefono': '',
                                'email': '',
                                'horario_desde': '09:00',
                                'horario_hasta': '19:00',
                                'fecha_recogida': getToday(),
                                'observaciones': '',
                            };

                            var data = {
                                'function': 'generarEnvioRapid',
                                'orderId': '<?php echo $order->get_id() ?>',
                                'producto': $("#rapid_producto").val(),
                                'destino': destino,
                                'recogida': recogida 
                            };

                            $.ajax({
                                type: "POST",
                                url: '../wp-content/plugins/rapidexpress_wordpress/functions.php',
                                data: data,
                                context: this,
                                success: function(success){
                                    if(JSON.parse(success).result == 200 && JSON.parse(success).albaran != ''){
                                        location.reload();
                                    } else {
                                        var msg = JSON.parse(success);                                    
                                        alert(msg.error.error);
                                        this.disabled = false;
                                    }                                                                        
                                },
                            });
                        });

                        

                        jQuery('#obtenerAlbaran').on('click',function(e){                            

                            var data = {
                                'apiToken': '<?php echo base64_encode($token) ?>',
                                'function': 'obtenerAlbaran',
                                'orderId': '<?php echo $order->get_id() ?>',
                            };

                            $.ajax({
                                    type: "POST",
                                    url: '../wp-content/plugins/rapidexpress_wordpress/functions.php',
                                    data: data,
                                    success: function(success){
                                        window.open('/wp-content/plugins/rapidexpress_wordpress/albaranes/'+success+'.pdf', '_blank', 'fullscreen=yes');
                                    },
                            });

                        });

                        function getToday(){
                            var today = new Date();
                            var dd = String(today.getDate()).padStart(2, '0');
                            var mm = String(today.getMonth() + 1).padStart(2, '0');
                            var yyyy = today.getFullYear();
                            return yyyy + '-' + mm + '-' + dd;
                        }

                    });
                </script> <?php
            }
        }
    }

    new Rapidexpress_Plugin();

?>